// *************************************************************************************
// SCICHART® Copyright SciChart Ltd. 2011-2020. All rights reserved.
//  
// Web: http://www.scichart.com
//   Support: support@scichart.com
//   Sales:   sales@scichart.com
// 
// CreateMultiPaneStockChartsViewModel.cs is part of the SCICHART® Examples. Permission is hereby granted
// to modify, create derivative works, distribute and publish any part of this source
// code whether for commercial, private or personal use. 
// 
// The SCICHART® examples are distributed in the hope that they will be useful, but
// without any warranty. It is provided "AS IS" without warranty of any kind, either
// expressed or implied. 
// *************************************************************************************
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Windows.Input;
using SciChart.Charting;
using SciChart.Charting.ChartModifiers;
using SciChart.Charting.Common.Helpers;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.ViewportManagers;
using SciChart.Charting.Visuals.TradeChart;
using SciChart.Data.Model;
using SciChart.Examples.ExternalDependencies.Common;
using SciChart.Examples.ExternalDependencies.Data;

namespace SciChart.Examples.Examples.CreateStockCharts.MultiPane
{
    public class CreateMultiPaneStockChartsViewModel : BaseViewModel
    {
        private IndexRange _xAxisVisibleRange;
        private ObservableCollection<BaseChartPaneViewModel> _chartPaneViewModels = new ObservableCollection<BaseChartPaneViewModel>();
        private readonly ICommand _closePaneCommand;
        private bool _isPanEnabled;
        private bool _isZoomEnabled;
        private string _verticalChartGroupId;
        private IViewportManager _viewportManager;
        Random random = new Random();
        private PriceSeries addLiveseries = null;
        public CreateMultiPaneStockChartsViewModel()
        {
            ZoomModeCommand = new ActionCommand(SetZoomMode);
            PanModeCommand = new ActionCommand(SetPanMode);
            ZoomExtentsCommand = new ActionCommand(ZoomExtends);
            AddPanelCommand = new ActionCommand(AddNewPanel);
            AddSelectionCommand = new ActionCommand(AddSelection);

            // Get prices and append for the main price chart (Candlestick)
            PriceSeries instrumentPriceData = null;

            //instrumentPriceData = DataManager.Instance.GetPriceData(Instrument.EurUsd.Value, TimeFrame.Daily);
            int maxbarscount = 10000;
            GetOHLCData(ref instrumentPriceData, maxbarscount);
            GetAdustRandomDataToMaxBarCount(ref instrumentPriceData, maxbarscount);

            addLiveseries = instrumentPriceData;


            // ChartGroup is an ID which is used to synchronize chart pane heights and mouse events. it must be unique per SciChartGroup, but differ if you have many top level SciChartGroups
            _verticalChartGroupId = Guid.NewGuid().ToString();

            _viewportManager = new DefaultViewportManager();
            var closePaneCommand = new ActionCommand<IChildPane>(pane => ChartPaneViewModels.Remove((BaseChartPaneViewModel)pane));

            _chartPaneViewModels.Add(new PricePaneViewModel(this, instrumentPriceData) { IsFirstChartPane = true, ViewportManager = _viewportManager, IsLastChartPane = true, Title= "Tadawul All Share Index" });
            //_chartPaneViewModels.Add(new MacdPaneViewModel(this, instrumentPriceData) { Title = "MACD", ClosePaneCommand = closePaneCommand });
            //_chartPaneViewModels.Add(new RsiPaneViewModel(this, instrumentPriceData) { Title = "RSI", ClosePaneCommand = closePaneCommand });
            //_chartPaneViewModels.Add(new VolumePaneViewModel(this, instrumentPriceData) { Title = "Volume", ViewportManager = _viewportManager, ClosePaneCommand = closePaneCommand, IsFirstChartPane = true, IsLastChartPane = true});

            SetZoomMode();
        }

       
       
        private void GetAdustRandomDataToMaxBarCount(ref PriceSeries instrumentPriceData, int cutoff)
        {
            int counter = 2;
            if (instrumentPriceData == null || instrumentPriceData.Count <= 0)
                instrumentPriceData = DataManager.Instance.GetPriceData(Instrument.EurUsd.Value, TimeFrame.Daily);

            //instrumentPriceData = new PriceSeries(instrumentPriceData.Count * counter);
            DateTime lastdate = instrumentPriceData[instrumentPriceData.Count - 1].DateTime;
            int orginalCount = instrumentPriceData.Count;
            
            //for (int i = 0; i < counter * orginalCount; i++)
            for (int i = 0; i < cutoff - orginalCount; i++)
            {
                PriceBar bar = new PriceBar();

                int index = i % orginalCount;
                bar.Open = instrumentPriceData[index].Open;
                bar.High = instrumentPriceData[index].High;
                bar.Low = instrumentPriceData[index].Low;
                bar.Close = instrumentPriceData[index].Close;
                bar.Volume = instrumentPriceData[index].Volume;
                lastdate = instrumentPriceData[instrumentPriceData.Count - 1].DateTime;

                NewDate:
                bar.DateTime = lastdate.AddDays(1);
                if (bar.DateTime.DayOfWeek == DayOfWeek.Saturday || bar.DateTime.DayOfWeek == DayOfWeek.Friday)
                {
                    lastdate = bar.DateTime;
                    goto NewDate;
                }

                instrumentPriceData.Add(bar);
            }
        }

        private void GetOHLCData(ref PriceSeries series, int cutoff)
        {
            LoadData(ref series, @"TASI.csv", cutoff);
        }

        private void LoadData(ref PriceSeries instrumentPriceData, string fileName, int cutoff)
        {
            //instrumentPriceData.Clear();
            instrumentPriceData = new PriceSeries();
            try
            {
                string fname;
                if (fileName.IndexOf(":\\") != -1)
                    fname = fileName;
                else
                    fname = AppDomain.CurrentDomain.BaseDirectory + fileName;

                using (FileStream fs = new FileStream(fname, FileMode.Open))
                {
                    using (StreamReader sr = new StreamReader(fs))
                    {
                        sr.BaseStream.Seek(0, SeekOrigin.Begin);
                        string line;
                        while ((line = sr.ReadLine()) != null && !line.Contains(",,,"))
                        {
                            PriceBar bar;
                            {
                                string[] record = line.Split(new[] { ',' });
                                string[] dateTime = record[0].Split(' ');
                                string[] date = dateTime[0].Split('/');
                                int hour = 0;
                                int minute = 0;
                                int second = 0;
                                if (dateTime.Length > 1)
                                {
                                    string[] time = dateTime[1].Split(':');
                                    hour = int.Parse(time[0], CultureInfo.InvariantCulture);
                                    minute = int.Parse(time[1], CultureInfo.InvariantCulture);
                                    if (time.Length > 2)
                                    {
                                        second = int.Parse(time[2], CultureInfo.InvariantCulture);
                                    }
                                }
                                bar = new PriceBar
                                {
                                    DateTime =
                                    new DateTime(int.Parse(date[2]),
                                                 int.Parse(date[0]),
                                                 Math.Min(int.Parse(date[1]), 31),
                                                 hour,
                                                 minute,
                                                 second),
                                    Open = double.Parse(record[1]),
                                    High = double.Parse(record[2]),
                                    Low = double.Parse(record[3]),
                                    Close = double.Parse(record[4]),
                                };
                                if (record.Length > 5)
                                {
                                    bar.Volume = long.Parse(record[5]);
                                }
                            }

                            if (instrumentPriceData.Count <= cutoff)
                                instrumentPriceData.Add(bar);
                            else
                                return;
                        }
                    }
                }
            }
            catch
            {
                return;
            }
        }

        private void AddSelection()
        {
            for (int i = 0; i < _chartPaneViewModels.Count; i++)
            {
                if (_chartPaneViewModels[i] != null)
                {
                    if (_chartPaneViewModels[i].ChartSeriesViewModels != null)
                    {
                        for (int j = 0; j < _chartPaneViewModels[i].ChartSeriesViewModels.Count; j++)
                        {
                            var series = _chartPaneViewModels[i].ChartSeriesViewModels[j];
                            series.IsSelected = true;
                        }
                    }
                }
            }
        }


        private void AddNewPanel()
        {
            int count = _chartPaneViewModels.Count - 2;
            if (count < 0)
                count = 0;
            _chartPaneViewModels.Insert(count, new VolumePaneViewModel(this, addLiveseries) { Title = "Volume", ClosePaneCommand = new ActionCommand<IChildPane>(pane => ChartPaneViewModels.Remove((BaseChartPaneViewModel)pane)), IsLastChartPane = false});
        }

        private void ZoomExtends()
        {
           // _viewportManager.AnimateZoomExtents(TimeSpan.FromMilliseconds(500));
        }

        public IEnumerable<string> AllThemes { get { return ThemeManager.AllThemes; } }

        public ICommand ZoomModeCommand { get; private set; }

        public ICommand PanModeCommand { get; private set; }

        public ICommand ZoomExtentsCommand { get; private set; }

        public ICommand AddPanelCommand { get; private set; }

        public ICommand AddSelectionCommand { get; private set; }

        public string VerticalChartGroupId
        {
            get { return _verticalChartGroupId; }
            set
            {
                if (_verticalChartGroupId == value) return;
                _verticalChartGroupId = value;
                OnPropertyChanged("VerticalChartGroupId");
            }
        }    

        /// <summary>
        /// Shared XAxis VisibleRange for all charts
        /// </summary>
        public IndexRange XVisibleRange
        {
            get { return _xAxisVisibleRange; }
            set
            {
                if (Equals(_xAxisVisibleRange, value)) return;

                //if (instrumentPriceData != null)
                //_xAxisVisibleRange = new IndexRange(0, instrumentPriceData.Count);
                //else
                _xAxisVisibleRange = value;

                OnPropertyChanged("XVisibleRange");
            }
        }        

        public ObservableCollection<BaseChartPaneViewModel> ChartPaneViewModels
        {
            get { return _chartPaneViewModels; }
            set
            {
                if (_chartPaneViewModels == value) return;

                _chartPaneViewModels = value;
                OnPropertyChanged("ChartPaneViewModels");
            }
        }

        public bool IsPanEnabled
        {
            get { return _isPanEnabled; }
            set
            {
                _isPanEnabled = value;
                OnPropertyChanged("IsPanEnabled");
            }
        }

        public bool IsZoomEnabled
        {
            get { return _isZoomEnabled; }
            set
            {
                _isZoomEnabled = value;
                OnPropertyChanged("IsZoomEnabled");
            }
        }


        private ChartModifierBase _chartModifier;
        public ChartModifierBase ChartModifier
        {
            get { return _chartModifier; }
            set
            {
                _chartModifier = value;
                OnPropertyChanged("ChartModifier");
            }
        }
        private void SetPanMode()
        {
            IsPanEnabled = true;
            IsZoomEnabled = false;
        }

        private void SetZoomMode()
        {
            IsPanEnabled = false;
            IsZoomEnabled = true;
        }
    }
}
