// *************************************************************************************
// SCICHART® Copyright SciChart Ltd. 2011-2020. All rights reserved.
//  
// Web: http://www.scichart.com
//   Support: support@scichart.com
//   Sales:   sales@scichart.com
// 
// PricePaneViewModel.cs is part of the SCICHART® Examples. Permission is hereby granted
// to modify, create derivative works, distribute and publish any part of this source
// code whether for commercial, private or personal use. 
// 
// The SCICHART® examples are distributed in the hope that they will be useful, but
// without any warranty. It is provided "AS IS" without warranty of any kind, either
// expressed or implied. 
// *************************************************************************************
using System;
using System.Windows.Media;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.RenderableSeries;
using SciChart.Examples.ExternalDependencies.Data;

namespace SciChart.Examples.Examples.CreateStockCharts.MultiPane
{
    public class PricePaneViewModel : BaseChartPaneViewModel
    {
        public PricePaneViewModel(CreateMultiPaneStockChartsViewModel parentViewModel, PriceSeries prices)
            : base(parentViewModel)
        {
            // We can add Series via the SeriesSource API, where SciStockChart or SciChartSurface bind to IEnumerable<IChartSeriesViewModel>
            // Alternatively, you can delcare your RenderableSeries in the SciStockChart and just bind to DataSeries
            // A third method (which we don't have an example for yet, but you can try out) is to create an Attached Behaviour to transform a collection of IDataSeries into IRenderableSeries

            // Add the main OHLC chart
            var stockPrices = new OhlcDataSeries<DateTime, double>() { SeriesName = "OHLC" };
            
            stockPrices.Append(prices.TimeData, prices.OpenData, prices.HighData, prices.LowData, prices.CloseData);
            ChartSeriesViewModels.Add(new CandlestickRenderableSeriesViewModel
            {

                //DataSeries = stockPrices,
                AntiAliasing = false,
                //StrokeUp = Colors.CadetBlue,
                //StrokeDown = Colors.Orange,
                //FillDown = new SolidColorBrush( Colors.Transparent),
                //FillUp = new SolidColorBrush(Colors.Transparent),
            });

            ChartSeriesViewModels[0].DataSeries = null;
            ChartSeriesViewModels[0].DataSeries = stockPrices;

            // Add a moving average
            //var maLow = new XyDataSeries<DateTime, double>() { SeriesName = "Low Line" };
            //maLow.Append(prices.TimeData, prices.CloseData.MovingAverage(50));
            //ChartSeriesViewModels.Add(new LineRenderableSeriesViewModel
            //{
            //    DataSeries = maLow,
            //    StyleKey = "LowLineStyle",
            //});

            //// Add a moving average
            //var maHigh = new XyDataSeries<DateTime, double>() { SeriesName = "High Line" };
            //maHigh.Append(prices.TimeData, prices.CloseData.MovingAverage(200));
            //ChartSeriesViewModels.Add(new LineRenderableSeriesViewModel
            //{
            //    DataSeries = maHigh,
            //    StyleKey = "HighLineStyle",
            //});


            //Try Bollinger band (dummy test)
            // Set a DataSeries of type x=DateTime, y0=Double, y1=double on the RenderableSeries declared in XAML
            //var bandseries = new XyyDataSeries<DateTime, double>();
            //ChartSeriesViewModels.Add(new BandRenderableSeriesViewModel
            //{
            //    Fill = Colors.Navy,
            //    FillY1 = Colors.Navy,
            //    DataSeries = bandseries,
            //    Opacity = 0.4,
            //    Stroke = Colors.DarkBlue,
            //    StrokeY1 = Colors.DarkRed
            //});

            // Get some data for the upper and lower band
            //var data = maLow;
            //var moreData = maHigh;

            //for (int i = 0; i < prices.Count; i++)
            //{

            //    // Append data to series. SciChart automatically redraws
            //    bandseries.Append(
            //        prices[i].DateTime,
            //        prices[i].Close - 0.05,
            //        prices[i].Close + 0.05);
            //}


            //old setting
            YAxisTextFormatting = "$0.0000";

            //YAxisTextFormatting = "###E+0";
        }
    }
}
